import subprocess
import click

import requests
from .ChatGPT import Chatbot
import os
import sys
from rich.live import Live
from rich.markdown import Markdown
from .code_execution import *

sys.stdin.reconfigure(encoding='utf-8')

@click.command()
def hello():
    """Say hello"""
    click.echo('Hello, world!')

@click.group()
def dxr():
    """Figlu command line tool"""
    # 默认调用bash函数
    pass

def get_bash_scripts():
    url = 'http://39.101.69.111:4000/bash'
    r = requests.get(url)
    scripts = r.json()
    return scripts

def get_bash_script_dir(script):
    url = f'http://39.101.69.111:4000/bash/{script}'
    r = requests.get(url)
    scripts = r.json()
    return scripts

def get_bash_script(script, sub_script):
    url = f'http://39.101.69.111:4000/bash/{script}/{sub_script}'
    r = requests.get(url)
    script = r.text
    return script

def chatgpt_explanation(language, error_message, model, maxtoken):
    query = construct_query(language, error_message)
    base_url, api_key = get_api_key()
    chatbot = Chatbot(api_key=api_key, base_url=base_url, engine=model, max_tokens=maxtoken)
    res = chatbot.ask_stream(query)
    return res

def choose_script(scripts, step=1):
    script_index = 0
    while True:
        click.clear()
        click.echo('Select a script:')
        for i, script in enumerate(scripts):
            if i == script_index:
                click.echo(f'> {script}')
            else:
                click.echo(f'  {script}')
        key = click.getchar()
        # 上下键选择，左键返回，右键进入
        if key == '\r':
            return scripts[script_index], step + 1
        elif key == '\x1b[A':
            script_index = (script_index - 1) % len(scripts)
        elif key == '\x1b[B':
            script_index = (script_index + 1) % len(scripts)
        elif key == '\x1b[D':
            return None, step - 1
        elif key == '\x1b[C':
            return scripts[script_index], step + 1
            

@dxr.command()
def bash():
    """执行云端上的脚本"""
    import os
    # scripts = get_bash_scripts()
    # script = choose_script(scripts)
    # sub_scripts = get_bash_script_dir(script)
    # sub_script = choose_script(sub_scripts)
    # bash_script = get_bash_script(script, sub_script)
    # print("=" * 80)
    # click.echo(bash_script)
    # print("=" * 80)
    # click.echo('Run this script?')
    # if click.confirm('Continue?'):
    #     os.system(bash_script)
    step = 1
    while True:
        if step == 0:
            break
        if step == 1:
            scripts = get_bash_scripts()
            script, step = choose_script(scripts, step)
        elif step == 2:
            sub_scripts = get_bash_script_dir(script)
            sub_script, step = choose_script(sub_scripts, step)
        elif step == 3:
            bash_script = get_bash_script(script, sub_script)
            print("=" * 80)
            click.echo(bash_script)
            print("=" * 80)
            click.echo('Run this script?')
            if click.confirm('Continue?'):
                os.system(bash_script)
                break
            step = 2
    
def get_api_key():
    api_key = os.environ.get("OPENAI_API_KEY", '')
    if api_key == '':
        url = f'http://39.101.69.111:4000/chat/get_api_key'
        r = requests.get(url)
        base_url_and_api_key = r.text
        base_url, api_key = base_url_and_api_key.split(',')
    return base_url, api_key  

@click.command()
@click.option("--model", default="gpt-3.5-turbo", help="Specify which GPT model to use")
@click.option("--maxtoken", default=4096, help="Maximum number of tokens in a single prompt")
def chat(model, maxtoken):
    """Chat with GPT-3 or GPT-4"""
    base_url, api_key = get_api_key()
    bot = Chatbot(api_key=api_key, base_url=base_url, engine=model, max_tokens=maxtoken)
    while True:
        text = input("You: ")
        if text.strip() == "exit":
            break
        response = bot.ask_stream(text)
        md = Markdown("")
        with Live(md, auto_refresh=False) as live:
            tmp = ""
            for r in response:
                tmp += r
                md = Markdown(tmp)
                live.update(md, refresh=True)
                

@click.command()
@click.option("--model", default="gpt-3.5-turbo", help="Specify which GPT model to use")
@click.option("--maxtoken", default=4096, help="Maximum number of tokens in a single prompt")
def git(model, maxtoken):
    """问问关于git的问题"""
    print("这是一个git命令行助手,你可以通过这个助手来学习git命令")
    # use openai to generate git scripts
    base_url, api_key = get_api_key()
    bot = Chatbot(api_key=api_key, base_url=base_url, system_prompt="""
    你是一个很好的git命令行助手，你可以帮助我更好的使用git, 
    根据我的问题,你可以给我提供一些git命令
    """, engine=model, max_tokens=maxtoken)
    while True:
        text = input("请输入你的问题: ")
        if text.strip() == "exit":
            break
        response = bot.ask_stream(text)
        print("=" * 80)
        tmp = ""
        for r in response:
            tmp += r
            print(r, end='', flush=True)
        print()
        print("=" * 80)
    
# dxr python test.py --model gpt-3.5-turbo --maxtoken 4096 
# 用来直接运行python脚本，并且获取运行时的错误输出，
# 将错误输出 发送给openai，让openai来帮助我们解决这个问题  
@click.command()
@click.option("--model", default="gpt-3.5-turbo", help="Specify which GPT model to use")
@click.option("--maxtoken", default=4096, help="Maximum number of tokens in a single prompt")
@click.option("--binary", default="python", help="Specify which binary to use")
@click.argument('script', type=click.Path(exists=True))
def python(model, maxtoken, binary,  script):
    args = [binary, script]
    language = binary
    if not language:
        print_invalid_language_message()
        return
    error_message = execute_code(args, language)
    if not error_message:
        return
    with LoadingMessage():
        res = chatgpt_explanation(language, error_message, model, maxtoken)
    md = Markdown("")
    with Live(md, auto_refresh=True) as live:
        tmp = ""
        for r in res:
            tmp += r
            md = Markdown(tmp)
            live.update(md, refresh=True)
            
            
@click.command()
@click.option("--model", default="gpt-3.5-turbo", help="Specify which GPT model to use")
@click.option("--maxtoken", default=4096, help="Maximum number of tokens in a single prompt")
@click.argument('script', type=click.Path(exists=True))
def python3(model, maxtoken, script):
    args = ['python3', script]
    language = get_language(args)
    print(language)
    if not language:
        print_invalid_language_message()
        return
    error_message = execute_code(args, language)
    if not error_message:
        return
    with LoadingMessage():
        res = chatgpt_explanation(language, error_message, model, maxtoken)
    md = Markdown("")
    with Live(md, auto_refresh=True) as live:
        tmp = ""
        for r in res:
            tmp += r
            md = Markdown(tmp)
            live.update(md, refresh=True)
            
            
            
@click.command()
@click.argument('args', nargs=-1)
@click.option("--model", default="gpt-3.5-turbo", help="Specify which GPT model to use")
@click.option("--maxtoken", default=4096, help="Maximum number of tokens in a single prompt")
@click.option("--lines", default=100, help="Number of lines before and after the error to capture as context")
def run(args, model, maxtoken, lines):
    """
    接收不固定参数的命令，并执行它们。如果发生错误，将捕获并显示错误输出及其上下文。
    """
    try:
        language = get_language(args)
        # 将参数元组转换为列表，并将其传递给subprocess.run函数
        result = subprocess.run(
            list(args), stderr=subprocess.PIPE, stdout=subprocess.PIPE, text=True, check=True
        )
    except subprocess.CalledProcessError as e:
        # 找出可能涉及的文件
        source_files = [arg for arg in args if os.path.isfile(arg)]
        file_contents = []

        for file in source_files:
            # 检查文件字数是否超过2000
            num_chars = os.stat(file).st_size
            if num_chars <= 2000:
                with open(file, 'r', encoding='utf-8') as f:
                    content = f.read()
            else:
                # 取前后各100行的代码
                # with open(file, 'r', encoding='utf-8') as f:
                #     head_lines = [next(f) for _ in range(lines)]
                # with open(file, 'r', encoding='utf-8') as f:
                #     tail_lines = f.readlines()[-lines:]
                with open(file, 'r', encoding='utf-8') as f:
                    head_lines = [next(f, '') for _ in range(lines)]
                with open(file, 'r', encoding='utf-8') as f:
                    tail_lines = [line for _, line in zip(range(lines), f.readlines())]
                    

                content = "".join(head_lines) + "\n...\n" + "".join(tail_lines)

            file_contents.append(content)

        # 获取错误输出
        error_output = e.stderr
        
        # 打印错误输出
        click.echo(error_output)

        # 将文件内容与错误输出合并
        total_input = f"Error Output:\n{error_output}\n\n"
        for i, content in enumerate(file_contents):
            total_input += f"File Content ({source_files[i]}):\n{content}\n\n"

        # 使用GPT模型处理整个输入
        with LoadingMessage():
            response = chatgpt_explanation(language, total_input, model, maxtoken)

        # 使用Rich库将结果显示为Markdown
        md = Markdown("")
        with Live(md, auto_refresh=True) as live:
            tmp = ""
            for r in response:
                tmp += r
                md = Markdown(tmp)
                live.update(md, refresh=True)
    else:
        click.echo(result.stdout)
    
        
    
    

     
    


def main():
    print("Hello world!")

dxr.add_command(hello)
dxr.add_command(bash)
dxr.add_command(chat)
dxr.add_command(git)
dxr.add_command(python)
dxr.add_command(python3)
dxr.add_command(run)

if __name__ == '__main__':
    dxr()



import os
import zipfile
import asyncio
import grpc
import pathlib
from grpc import aio
from . import zip_service_pb2
from . import zip_service_pb2_grpc
import logging


class ZipService(zip_service_pb2_grpc.ZipServiceServicer):

    async def StartZip(self, request, context):
        directory_to_zip = request.dir
        print(f"Start zip directory: {directory_to_zip}")
        total_size = sum(f.stat().st_size for f in pathlib.Path(directory_to_zip).rglob('*') if f.is_file())
        zipped_size = 0
        loop = asyncio.get_event_loop()

        with zipfile.ZipFile('output.zip', 'w', zipfile.ZIP_DEFLATED) as zf:
            print(f"Total size: {total_size}")
            for dirpath, dirnames, filenames in os.walk(directory_to_zip):
                for filename in filenames:
                    file_path = os.path.join(dirpath, filename)
                    file_size = os.path.getsize(file_path)
                    await loop.run_in_executor(None, zf.write, file_path)
                    zipped_size += file_size
                    progress = zipped_size / total_size
                    yield zip_service_pb2.ZipProgress(progress=progress)

    async def DownloadZip(self, request, context):
        chunk_size = 1024 * 1024  # 1MB
        total_sent_size = 0

        with open('output.zip', 'rb') as f:
            file_size = os.path.getsize('output.zip')
            while True:
                read_data = f.read(chunk_size)
                total_sent_size += len(read_data)

                if len(read_data) == 0:
                    return
                
                yield zip_service_pb2.ZipChunk(data=read_data, total_size=file_size, sent_size=total_sent_size)

    @classmethod
    async def serve(cls):
        server = aio.server()
        zip_service_pb2_grpc.add_ZipServiceServicer_to_server(cls(), server)
        server.add_insecure_port('[::]:50051')
        print("Starting server. Listening on port 50051.")
        await server.start()
        await server.wait_for_termination()

    @classmethod
    def start_sync(cls):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            loop.run_until_complete(cls.serve())
        finally:
            loop.close()
            asyncio.set_event_loop(None)


if __name__ == "__main__":
    logging.basicConfig()
    ZipService.start_sync()



import redis
import sys
import traceback
import time
import numpy as np
import cv2
from loguru import logger

class RedisClient:
    _instance = None

    def __new__(cls, *args, **kwargs):
        # Singleton pattern
        if cls._instance is None:
            cls._instance = object.__new__(cls)
        return cls._instance

    def __init__(self, host, port, password=None):
        self.host = host
        self.port = port
        self.redis_conn = None
        try:
            self.pool = redis.ConnectionPool(host=self.host,
                                             port=self.port,
                                             decode_responses=True,
                                             db=0,
                                             password=password,
                                             max_connections=100)
            logger.info(f'获取Redis连接池, Host={self.host}, Port={self.port}')
        except Exception as e:
            logger.error("获取Redis连接池异常, 程序退出:{}Tracelog={}".format(str(e), traceback.format_exc()))
            sys.exit(0)

    def get_redis_client(self):
        try:
            if self.redis_conn is None or not self.redis_conn.ping():
                self.redis_conn = redis.StrictRedis(connection_pool=self.pool)

                if self.redis_conn.ping():
                    logger.success(f'获取Redis连接成功, Host={self.host}, Port={self.port}')
                self.redis_conn.flushall()
        except Exception as e:
            logger.error("Redis连接*异常*:{}, Tracelog={}".format(str(e), traceback.format_exc()))

    def get_redis_data(self, key):
        self.get_redis_client()
        try:
            res = self.redis_conn.get(key)
            redis_out = {'Frame': None, 'bFlag': False, 'error': False, 'iTargetNum': 0, 'iType': '', 'sType': ''}
            if res is not None:
                redis_data = eval(res)
                nparr = np.frombuffer(redis_data['Frame'], np.uint8)
                redis_out['Frame'] = cv2.imdecode(nparr, cv2.IMREAD_COLOR)
                result = redis_data.get('Result', {})
                i_type_list = [str(alg_type) for alg_type in result.keys()]
                s_type_list = [result[alg_type].get('sType', '') for alg_type in result.keys()]
                redis_out['iType'] = ','.join(i_type_list)
                redis_out['sType'] = ','.join(s_type_list)

                for algtype in result:
                    res_data = result[algtype]
                    redis_out['bFlag'] = redis_out['bFlag'] or res_data['bFlag']
                    redis_out['error'] = redis_out['error'] or res_data['error']
                    res_results = res_data.get('lResults', {})
                    rect = res_results.get('rect', [])
                    redis_out['iTargetNum'] += len(rect)
                    s_value = res_results.get('sValue', '')
                    if s_value:
                        redis_out['iTargetNum'] = s_value
            return redis_out
        except redis.exceptions.ConnectionError:
            logger.error("Redis连接已断开，重新连接中...")
            self.get_redis_client()
            return self.get_redis_data(key)
        except Exception as e:
            logger.error("获取Redis数据异常:{}, Tracelog={}".format(str(e), traceback.format_exc()))
            return None

    def gen_redis_data(self, key):
        while True:
            result = self.get_redis_data(key)
            if result is not None:
                yield result
            else:
                time.sleep(0.1)

if __name__ == '__main__':
    redis_client = RedisClient('10.10.8.43', 7777, '123456')
    gen_data = redis_client.gen_redis_data('cam_rtsp_left')

    for data in gen_data:
        if data.get('Frame', None) is not None:
            print(data['Frame'].shape)
        else:
            print('Frame data is None')
            time.sleep(0.1)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zip_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11zip_service.proto\x12\nzipservice\"\x19\n\nZipRequest\x12\x0b\n\x03\x64ir\x18\x01 \x01(\t\"\x1f\n\x0bZipProgress\x12\x10\n\x08progress\x18\x01 \x01(\x02\"#\n\x0f\x44ownloadRequest\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\"?\n\x08ZipChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x12\n\ntotal_size\x18\x02 \x01(\x03\x12\x11\n\tsent_size\x18\x03 \x01(\x03\x32\x8f\x01\n\nZipService\x12=\n\x08StartZip\x12\x16.zipservice.ZipRequest\x1a\x17.zipservice.ZipProgress0\x01\x12\x42\n\x0b\x44ownloadZip\x12\x1b.zipservice.DownloadRequest\x1a\x14.zipservice.ZipChunk0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'zip_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ZIPREQUEST']._serialized_start=33
  _globals['_ZIPREQUEST']._serialized_end=58
  _globals['_ZIPPROGRESS']._serialized_start=60
  _globals['_ZIPPROGRESS']._serialized_end=91
  _globals['_DOWNLOADREQUEST']._serialized_start=93
  _globals['_DOWNLOADREQUEST']._serialized_end=128
  _globals['_ZIPCHUNK']._serialized_start=130
  _globals['_ZIPCHUNK']._serialized_end=193
  _globals['_ZIPSERVICE']._serialized_start=196
  _globals['_ZIPSERVICE']._serialized_end=339
# @@protoc_insertion_point(module_scope)



import json
import paho.mqtt.client as mqtt
import threading
import types
import time

class MQTTManager():    
    def __init__(self, server_url, server_port=1883, client_id=None, heart_beat_time=60):
        self.server_url = server_url
        self.server_port = server_port
        self.client_id = client_id if client_id else str(int(time.time() * 1000))
        print(self.client_id)
        self.heart_beat_time = heart_beat_time
        self.mqttc = mqtt.Client(client_id=self.client_id)
        self.callbacks = {}
        self.lock = threading.Lock()  # 添加一个锁
        self.connect_client()

    def connect_client(self):
        self.mqttc.will_set('/topic/' + self.client_id, '', retain=True)
        self.mqttc.on_message = self.on_message
        self.mqttc.on_connect = self.on_connect
        self.mqttc.on_subscribe = self.on_subscribe
        self.mqttc.on_disconnect = self.on_disconnect
        self.mqttc.on_log = self.on_log
        self.mqttc.connect(self.server_url, self.server_port, self.heart_beat_time)
        threading.Thread(target=self.mqttc.loop_forever).start()

    def topic_callback(self, topic, func):
        with self.lock:
            print(f'self.client_id: {self.client_id}, topic: {topic}')
            self.callbacks[topic] = func
            self.mqttc.subscribe(topic)

    def on_connect(self, client, userdata, flags, rc):
        print("Connected with result code " + str(rc))

    def on_message(self, client, userdata, message):
        topic = message.topic
        with self.lock:
            if topic in self.callbacks:
                thread = threading.Thread(target=self.callbacks[topic], args=(self.client_id, message,), daemon=True)
                thread.start()

    def on_subscribe(self, client, userdata, mid, granted_qos):
        print(f"Subscribed: " + str(mid) + " " + str(granted_qos))

    def on_disconnect(self, client, userdata, rc):
        print("Disconnected")   

    def on_log(self, client, userdata, level, buf):
        # print(f'on_log: {buf}, client_id: {self.client_id}')
        pass

class MQTTManagerGroup():
    def __init__(self, server_info):
        self.managers = []

        for info in server_info:
            manager = MQTTManager(server_url=info['server_url'], client_id=info['client_id'])
            self.managers.append(manager)

    def topic_callback(self, topic):
        def decorator(f):
            for manager in self.managers:
                print(f'self.client_id: {manager.client_id}, topic: {topic}')
                manager.topic_callback(topic, f)  # 现在你可以直接传递回调函数
            return f
        return decorator
    
    def publish(self, client_id, topic, payload):
        for manager in self.managers:
            if manager.client_id == client_id:
                manager.mqttc.publish(topic, json.dumps(payload))
                return
        print(f"No client with id {client_id} found.")


if __name__ == '__main__':
    server_info = [
        {
            "server_url": "10.10.0.101",
            "client_id": "10.10.0.101",
        },
        {
            "server_url": "10.10.0.195",
            "client_id": "10.10.0.195",
        }
    ]
    group = MQTTManagerGroup(server_info)

    # 使用装饰器定义回调
    @group.topic_callback('/get_speed_response')
    def handle_message_1(client_id, message):
        json_str = message.payload.decode()
        data = json.loads(json_str)  # 假设消息总是JSON格式
        print(f"Received message from {client_id}: {data}")
        
    # 使用publish方法发布消息
    while True:
        time.sleep(1)
        group.publish("10.10.0.101", "/test", {"speed": 50})
        group.publish("10.10.0.195", "/test", {"speed": 75})


import asyncio
import time
import grpc
from grpc import aio
import zipfile
from . import zip_service_pb2
from . import zip_service_pb2_grpc
import logging     
import os
import threading

class ZipClient:
    def __init__(self, ip, zip_progress_callback, download_progress_callback, download_completion_callback=None):
        self.stub = None
        self.zip_progress_callback = zip_progress_callback
        self.download_progress_callback = download_progress_callback
        self.download_completion_callback = download_completion_callback
        self.ip = ip
        self.task = None  # this is to keep track of the task
        self.thread = None  # this is to keep track of the separate thread
        

    async def start(self, directory):
        # Create gRPC channel and stub
        channel = aio.insecure_channel(f'{self.ip}:50051')
        self.stub = zip_service_pb2_grpc.ZipServiceStub(channel)

        try:
            # Call StartZip
            request = zip_service_pb2.ZipRequest(dir=directory)
            async for response in self.stub.StartZip(request):
                self.zip_progress_callback(response.progress)

            # 获取用户目录
            user_dir = os.path.expanduser('~')
            # 要创建/检查的目录
            dxr_dir = os.path.join(user_dir, 'dxr')

            # 检查目录是否存在，如果不存在则创建
            if not os.path.exists(dxr_dir):
                os.makedirs(dxr_dir)

            # 你要写入的文件的最终路径
            file_path = os.path.join(dxr_dir, 'downloaded.zip')

            # 打开文件并准备写入数据
            with open(file_path, 'wb') as f:
                # Call DownloadZip
                request = zip_service_pb2.DownloadRequest(filename='output.zip')
                async for response in self.stub.DownloadZip(request):
                    if response.data:
                        # 将接收到的数据直接写入文件
                        f.write(response.data)
                        progress = response.sent_size / response.total_size
                        self.download_progress_callback(progress)
        except asyncio.exceptions.CancelledError:
            print('Download task cancelled.')
            self.download_completion_callback(-1)
        else:
            print('Download task completed.')
            if self.download_completion_callback:
                self.download_completion_callback(0)
        finally:
            await channel.close()
        
        
    def start_sync(self, directory):
        if self.thread is not None and self.thread.is_alive():
            print("Thread already running. Please wait for current operation to finish or stop.")
            return
        print("Starting download task.", directory)
        self.thread = threading.Thread(target = self._start_thread, args = (directory,))
        self.thread.start()
        
    def _start_thread(self, directory):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            self.task = loop.create_task(self.start(directory))  # creates a new task
            loop.run_until_complete(self.task)  # runs the task
        finally:
            loop.close()
            asyncio.set_event_loop(None)

    def stop_sync(self):
        if self.task:   # if a task exists, stop it
            try:
                self.task.cancel()
                print('Stopping download task.')
            except:
                print("Download task stop failed.")
        

def zip_progress_update(progress):
    print('Zip Progress: {:.2f}'.format(progress * 100))

def download_progress_update(progress):
    print('Download Progress: {:.2f}'.format(progress * 100))


# User code:
if __name__ == '__main__':
    logging.basicConfig()
    client = ZipClient(zip_progress_update, download_progress_update, ip="10.10.0.195")
    client.start_sync('/Users/luzhipeng/Desktop/Images/图片')
    
    time.sleep(10)
    
    client.stop_sync()



from Dxr_mqtt.dxr_mqtt_mulity import MQTTManagerGroup
import time, json

if __name__ == '__main__':
    server_info = [
        {
            "server_url": "10.10.0.101",
            "client_id": "10.10.0.101",
        },
        {
            "server_url": "10.10.0.195",
            "client_id": "10.10.0.195",
        }
    ]
    group = MQTTManagerGroup(server_info)

    # 使用装饰器定义回调
    @group.topic_callback('/get_speed_response')
    def handle_message_1(client_id, message):
        json_str = message.payload.decode()
        data = json.loads(json_str)  # 假设消息总是JSON格式
        print(f"Received message from {client_id}: {data}")
        
    # 使用publish方法发布消息
    while True:
        time.sleep(1)
        group.publish("10.10.0.101", "/test", {"speed": 50})
        group.publish("10.10.0.195", "/test", {"speed": 75})